<?php
/**
 * Licensing and Update Manager Class
 * 
 * @package      Licensing Update Manager
 * @author       Tim Carr
 * @version      3.0.0
 * @copyright    WP Zinc
 */
class LicensingUpdateManager {

    /**
     * Flag to determine if we've queried the remote endpoint
     * for updates. Prevents plugin update checks running
     * multiple times
     *
     * @since   1.0.0
     *
     * @var     boolean
     */
    public $update_check = false;

    /**
     * Constructor.
     *
     * @since   1.0.0
     * 
     * @param   object  $plugin    WordPress Plugin
     * @param   string  $endpoint  Licensing Endpoint
     */
    public function __construct( $plugin, $endpoint ) {

        // Plugin Details
        $this->plugin = $plugin;
        $this->endpoint = $endpoint;
        
        // Admin Notice
        $this->notice = new stdClass;
        
        if ( is_admin() ) {
            // Check if the licensing form has been submitted
            // If so, save its results before we check the license key validitiy
            if ( isset( $_POST[ $this->plugin->name ] ) && is_array( $_POST[ $this->plugin->name ] ) && array_key_exists( 'licenseKey', $_POST[ $this->plugin->name ] ) ) {
                update_option( $this->plugin->name . '_licenseKey', $_POST[ $this->plugin->name ]['licenseKey'] );
                
                // Force a license key check
                $this->check_license_key_valid( true );
            } else if ( isset( $_GET['page'] ) && $_GET['page'] == $this->plugin->name ) {
                // GET request on licensing screen
                // Force a license key check
                $this->check_license_key_valid( true );
            }
            
            // Hooks and Filters
            add_action( 'admin_notices', array( $this, 'admin_notices' ) );
            add_filter( 'plugins_api', array( $this, 'plugins_api' ), 10, 3 );
            add_action( 'after_plugin_row_' . $this->plugin->name . '/' . $this->plugin->name . '.php', array( $this, 'maybe_display_expiry_notice' ), 10, 3 );
        }

        // Check for updates, outside of is_admin() so WP-CLI is supported
        add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'api_check' ) );

        // Always perform a license check, so if the transient expires, a fresh check takes place to update the transient.
        $this->check_license_key_valid( false );

    }

    /**
     * Outputs Administration Notices relating to license key validation
     *
     * @since 3.0.0
     */
    public function admin_notices() {
        
        // If no message has been set, bail
        if ( ! isset( $this->notice->message ) ) {
            return false;
        }

        // Only show message if an error, or a success + we are on the licensing screen
        $screen = get_current_screen();
        if ( ( ! isset( $this->notice->error ) || $this->notice->error == 0 ) && $screen->base != 'toplevel_page_' . $this->plugin->name ) {
            // Success message, but we're not on the licensing screen - bail
            return false;
        }

        // Output message
        echo ( '<div class="notice ' . ( ( isset( $this->notice->error ) && $this->notice->error == 1 ) ? 'error' : 'updated' ) . '">
            <p>' . $this->notice->message . '</p>
        </div>' );
    
    }

    /**
     * Gets the license key from either the wp-config constant, or the options table
     *
     * @since 3.0.0
     *
     * @return string License Key
     */
    public function get_license_key() {

        // If the license key is defined in wp-config, use that
        if ( defined( strtoupper( $this->plugin->name ) . '_LICENSE_KEY' ) ) {
            $license_key = constant( strtoupper( $this->plugin->name ) . '_LICENSE_KEY' );
        } else {
            // Get from options table
            $license_key = get_option( $this->plugin->name . '_licenseKey' );
        }

        return $license_key;

    }
    
    /**
     * Checks whether a license key has been specified in the settings table.
     * 
     * @since 3.0.0
     *
     * @return bool License Key Exists
     */                   
    public function check_license_key_exists() {

        // If the license key is defined in wp-config, use that
        if ( defined( strtoupper( $this->plugin->name ) . '_LICENSE_KEY' ) ) {
            $license_key = constant( strtoupper( $this->plugin->name ) . '_LICENSE_KEY' );
        } else {
            // Get from options table
            $license_key = get_option( $this->plugin->name . '_licenseKey' );
        }
        
        // Return license key
        return ( ( isset( $license_key ) && trim( $license_key ) != '' ) ? true : false );
    
    }    
    
    /**
     * Checks whether the license key stored in the settings table exists and is valid.
     *
     * If so, we store the latest remote plugin details in a transient, which can then be used when
     * updating plugins.
     * 
     * @since 3.0.0
     *
     * @param bool $force   Force License Key Check (used when saving the licensing screen form options)
     * @return bool         License Key Valid
     */
    public function check_license_key_valid( $force = false ) { 

        return true;

    }  
    
    /**
     * Hooks into the plugin update check process, telling WordPress if a newer version of our
     * Plugin is available.
     *
     * @since   3.0.0
     *
     * @param   array   $transient  Transient
     * @return  array               Transient Plugin Data
     */
    public function api_check( $transient ) {

        // If we haven't called the licensing endpoint (which includes product update info),
        // do so now.
        if ( ! $this->update_check ) {
            $this->update_check = true;

            // If the license key isn't valid, bail
            if ( ! $this->check_license_key_valid( true ) ) {
                return $transient;
            }
        }

        // If here, we have a valid license and product update information
        $remote_version = get_site_transient( $this->plugin->name . '_version' );
        $remote_package = get_site_transient( $this->plugin->name . '_package' );

        if ( $remote_version > $this->plugin->version ) {
            // New version available - add to transient
            $response               = new stdClass;
            $response->new_version  = $remote_package->version;
            $response->slug         = $remote_package->slug;
            $response->plugin       = $this->plugin->name . '/' . $this->plugin->name . '.php';
            $response->url          = $remote_package->homepage;
            $response->package      = $remote_package->download_link;
            
            // Add response to transient array
            $transient->response[ $this->plugin->name . '/' . $this->plugin->name . '.php' ] = $response;
        }
   
        return $transient;

    }

    /**
     * Hooks into the plugins_api process, telling WordPress information about our plugin, such
     * as the WordPress compatible version and the changelog.
     *
     * @since 3.0.0
     *
     * @param object    $api    The original plugins_api object.
     * @param string    $action The action sent by plugins_api.
     * @param array     $args   Additional args to send to plugins_api.
     * @return object           New stdClass with plugin information on success, default response on failure.
     */
    public function plugins_api( $api, $action = '', $args = null ) {

        // Check if we are getting info for our plugin
        $plugin = ( 'plugin_information' == $action ) && isset( $args->slug ) && ( $this->plugin->name == $args->slug );
        if ( ! $plugin ) {
            return $api;
        }

        // Get remote package data from transients
        // This was populated by the update/license checks earlier
        $remote_package = get_site_transient( $this->plugin->name . '_package' );
        if ( ! $remote_package ) {
            return $api;
        }

        // Create a new stdClass object and populate it with our plugin information.
        $api                        = new stdClass;
        $api->name                  = $remote_package->name;
        $api->slug                  = $remote_package->slug;
        $api->plugin                = $this->plugin->name . '/' . $this->plugin->name . '.php';
        $api->version               = $remote_package->version;
        $api->author                = $remote_package->author;
        $api->author_profile        = $remote_package->author_profile;
        $api->requires              = $remote_package->requires;
        $api->tested                = $remote_package->tested;
        $api->last_updated          = date( 'Y-m-d H:i:s', $remote_package->last_updated );
        $api->homepage              = $remote_package->homepage;
        $api->sections['changelog'] = $remote_package->changelog;
        $api->download_link         = $remote_package->download_link;

        // Return the new API object with our custom data.
        return $api;

    }

    /**
     * Displays an inline notice on the Plugins screen if the license has expired.
     *
     * @since   1.0.0
     */
    public function maybe_display_expiry_notice( $plugin_file, $plugin_data, $status ) {

        // Check license key validity, to populate $this->notice
        $this->check_license_key_valid( true );

        // If no message was added to the notice object, bail
        if ( ! isset( $this->notice->message ) ) {
            return;
        }

        // If the message doesn't contain anything about an expired license, bail
        if ( strpos( $this->notice->message, 'expired' ) === false ) {
            return;
        }

        // Output message
        echo '</tr>
        <tr class="plugin-update-tr active">
            <td colspan="3" class="plugin-update colspanchange">
                <div class="update-message notice inline notice-error notice-alt">
                    <p>
                        ' . str_replace( $this->plugin->displayName . ': ', '', $this->notice->message ) . '
                    </p>
                </div>
            </td>';

    }

}
