<?php
/**
 * Handles Group Table actions
 * 
 * @package  Page_Generator_Pro
 * @author   Tim Carr
 * @version  2.0.2
 */
class Page_Generator_Pro_Groups_Table {

    /**
     * Holds the base class object.
     *
     * @since   2.0.2
     *
     * @var     object
     */
    public $base;

    /**
     * Stores the current Group the settings are defined for.
     *
     * @since   2.0.2
     *
     * @var     int
     */
    public $group_id = 0;

    /**
     * Constructor.
     *
     * @since   2.0.2
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Bulk Actions Dropdown
        add_filter( 'bulk_actions-edit-' . $this->base->get_class( 'post_type' )->post_type_name, array( $this, 'register_bulk_actions' ) ); 
        add_filter( 'handle_bulk_actions-edit-' . $this->base->get_class( 'post_type' )->post_type_name, array( $this, 'run_bulk_actions' ), 10, 3 ); 

        // WP_List_Table Columns
        add_filter( 'manage_edit-page-generator-pro_columns', array( $this, 'admin_columns' ) );
        add_action( 'manage_page-generator-pro_posts_custom_column', array( $this, 'admin_columns_output' ), 10, 2 );

        // WP_List_Table Row Actions
        add_filter( 'post_row_actions', array( $this, 'admin_row_actions' ) );

        // Run any row actions called from the WP_List_Table
        add_action( 'init', array( $this, 'run_row_actions' ) );

    }

    /**
     * Adds Bulk Action options to Group WP_List_Table
     *
     * @since   1.9.9
     *
     * @param   array   $actions    Registered Bulk Actions
     * @return  array               Registered Bulk Actions
     */
    public function register_bulk_actions( $actions ) {

        // Define Actions
        $bulk_actions = array(
            'duplicate'                 => $this->base->get_class( 'groups_ui' )->get_title( 'duplicate' ),
            'generate_cron'             => $this->base->get_class( 'groups_ui' )->get_title( 'generate_cron' ),
            'trash_generated_content'   => $this->base->get_class( 'groups_ui' )->get_title( 'trash_generated_content' ),
            'delete_generated_content'  => $this->base->get_class( 'groups_ui' )->get_title( 'delete_generated_content' ),
        );

        // Remove some actions we don't want
        unset( $actions['edit'] );

        /**
         * Defines Bulk Actions to be added to the select dropdown on the Groups WP_List_Table.
         *
         * @since   1.9.9
         *
         * @param   array   $bulk_actions   Plugin Specific Bulk Actions
         * @param   string  $actions        Existing Registered Bulk Actions (excluding Plugin Specific Bulk Actions)
         */
        $bulk_actions = apply_filters( 'page_generator_pro_groups_table_register_bulk_actions', $bulk_actions, $actions );

        // Merge with default Bulk Actions
        $actions = array_merge( $bulk_actions, $actions );

        // Return
        return $actions;

    }

    /**
     * Handles Bulk Actions when one is selected to run
     *
     * @since   1.9.9
     *
     * @param   string  $redirect_to    Redirect URL
     * @param   string  $action         Bulk Action to Run
     * @param   array   $post_ids       Post IDs to apply Action on
     * @return  string                  Redirect URL
     */
    public function run_bulk_actions( $redirect_to, $action, $post_ids ) {

        // Bail if the action isn't specified
        if ( empty( $action ) ) {
            return $redirect_to;
        }

        // Bail if no Post IDs
        if ( empty( $post_ids ) ) {
            return $redirect_to;
        }

        // Setup notices class, enabling persistent storage
        $this->base->get_class( 'notices' )->enable_store();
        $this->base->get_class( 'notices' )->set_key_prefix( 'page_generator_pro_' . wp_get_current_user()->ID );

        // Run Bulk Action
        switch ( $action ) {

            /**
             * Duplicate
             */
            case 'duplicate':
                foreach ( $post_ids as $post_id ) {
                    // Duplicate
                    $result = $this->base->get_class( 'groups' )->duplicate( $post_id );

                    // If an error occured, add it to the notices
                    if ( is_wp_error( $result ) ) {
                        $this->base->get_class( 'notices' )->add_error_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $result->get_error_message() ) );
                    } else {
                        $this->base->get_class( 'notices' )->add_success_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $this->base->get_class( 'groups_ui' )->get_title( 'duplicate_success' ) ) );
                    }
                }
                break;

            /**
             * Generate via CRON
             */
            case 'generate_cron':
                foreach ( $post_ids as $post_id ) {
                    // Duplicate
                    $result = $this->base->get_class( 'groups' )->schedule_generation( $post_id );

                    // If an error occured, add it to the notices
                    if ( is_wp_error( $result ) ) {
                        $this->base->get_class( 'notices' )->add_error_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $result->get_error_message() ) );
                    } else {
                        $this->base->get_class( 'notices' )->add_success_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $this->base->get_class( 'groups_ui' )->get_title( 'generate_cron_success' ) ) );
                    }
                }
                break;

            /**
             * Trash Generated Content
             */
            case 'trash_generated_content':
                foreach ( $post_ids as $post_id ) {
                    // Duplicate
                    $result = $this->base->get_class( 'groups' )->trash_generated_content( $post_id );

                    // If an error occured, add it to the notices
                    if ( is_wp_error( $result ) ) {
                        $this->base->get_class( 'notices' )->add_error_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $result->get_error_message() ) );
                    } else {
                        $this->base->get_class( 'notices' )->add_success_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $this->base->get_class( 'groups_ui' )->get_title( 'generate_trash_generated_content_success' ) ) );
                    }
                }
                break;

            /**
             * Delete Generated Content
             */
            case 'delete_generated_content':
                foreach ( $post_ids as $post_id ) {
                    // Duplicate
                    $result = $this->base->get_class( 'groups' )->delete_generated_content( $post_id );

                    // If an error occured, add it to the notices
                    if ( is_wp_error( $result ) ) {
                        $this->base->get_class( 'notices' )->add_error_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $result->get_error_message() ) );
                    } else {
                        $this->base->get_class( 'notices' )->add_success_notice( sprintf( __( 'Group #%s: %s', 'page-generator-pro' ), $post_id, $this->base->get_class( 'groups_ui' )->get_title( 'generate_delete_generated_content_success' ) ) );
                    }
                }
                break;

            /**
             * Other Bulk Actions
             */
            default:
                /**
                 * Runs the given Bulk Action against the given Content Group IDs.
                 *
                 * @since   1.9.9
                 *
                 * @param   string  $action     Bulk Action Run
                 * @param   array   $post_ids   Group IDs
                 */
                do_action( 'page_generator-pro_groups_table_run_bulk_actions', $action, $post_ids );
                break;

        }

        // Return redirect
        return $redirect_to;

    }

    /**
     * Adds columns to the Groups CPT within the WordPress Administration List Table
     * 
     * @since   1.2.3
     *
     * @param   array   $columns    Columns
     * @return  array               New Columns
     */
    public function admin_columns( $columns ) {

        // Inject columns
        $new_columns = array(
            'cb'                    => '<input type="checkbox" />',
            'title'                 => __( 'Title' ),
            'description'           => __( 'Description' ),
            'type'                  => __( 'Content Type', 'page-generator-pro' ),
            'generated_count'       => __( 'No. Generated Items', 'page-generator-pro' ),
            'status'                => __( 'Status', 'page-generator-pro' ),
            'date'                  => __( 'Date' ),
        );

        /**
         * Filters the columns to display on the Groups: Content WP_List_Table.
         *
         * @since   1.2.3
         *
         * @param   array   $new_columns    New Columns
         * @param   array   $columns        Columns
         */
        $new_columns = apply_filters( 'page_generator_pro_groups_table_admin_columns', $new_columns, $columns );

        // Footer
        add_action( 'admin_footer', array( $this, 'output_js_overlay' ) );

        return $new_columns;

    }


    /**
     * Manages the data to be displayed within a column on the Groups CPT within 
     * the WordPress Administration List Table
     * 
     * @since   1.2.3
     *
     * @param   string  $column_name    Column Name
     * @param   int     $post_id        Group Post ID
     */
    public function admin_columns_output( $column_name, $post_id ) {

        // Array to hold the item(s) to output in this column for this Group
        $items = array();

        // Get group settings, if we don't have them
        if ( $post_id != $this->group_id ) {
            $this->group_id = $post_id;
            $this->settings = $this->base->get_class( 'groups' )->get_settings( $this->group_id );
        }

        switch ( $column_name ) {
            /**
             * Description
             */
            case 'description':
                $items = array(
                    'description' => nl2br( $this->settings['description'] ),
                );
                break;

            /**
             * Type
             */
            case 'type':
                $items = array(
                    'type' => $this->settings['type'],
                );
                break;

            /**
             * Number of Generated Pages
             */
            case 'generated_count':
                if ( $this->base->get_class( 'groups' )->has_generated_content( $post_id ) ) {
                    $items = array(
                        'generated_count'           => $this->settings['generated_pages_count'],
                        'trash_generated_content'   => '<a href="edit.php?post_type=' . $this->base->get_class( 'post_type' )->post_type_name . '&' . $this->base->plugin->name . '-action=trash_generated_content&id=' . $post_id . '&type=content">' . __( 'Trash Generated Content', 'page-generator-pro' ) . '</a>',
                        'delete_generated_content'  => '<a href="edit.php?post_type=' . $this->base->get_class( 'post_type' )->post_type_name . '&' . $this->base->plugin->name . '-action=delete_generated_content&id=' . $post_id . '&type=content">' . __( 'Delete Generated Content', 'page-generator-pro' ) . '</a>',
                    );
                } else {
                    $items = array(
                        'generated_count' => 0,
                    );
                }
                break;

            /**
             * Status
             */
            case 'status':
                if ( $this->base->get_class( 'groups' )->is_idle( $post_id ) ) {
                    $items = array(
                        'test'          => '<a href="edit.php?post_type=' . $this->base->get_class( 'post_type' )->post_type_name . '&' . $this->base->plugin->name . '-action=test&id=' . $post_id . '&type=content">' . $this->base->get_class( 'groups_ui' )->get_title( 'test' ) . '</a>',
                        'generate'      => '<a href="admin.php?page=' . $this->base->plugin->name . '-generate&id=' . $post_id . '&type=content">' . $this->base->get_class( 'groups_ui' )->get_title( 'generate' ) . '</a>',
                        'generate_cron' => '<a href="edit.php?post_type=' . $this->base->get_class( 'post_type' )->post_type_name . '&' . $this->base->plugin->name . '-action=generate_cron&id=' . $post_id . '&type=content">' . $this->base->get_class( 'groups_ui' )->get_title( 'generate_cron' ) . '</a>',
                    );
                } else {
                    $items = array(
                        'status'            => '<div class="page-generator-pro-generating-spinner">
                                                <span class="spinner"></span>' . 
                                                sprintf( 
                                                    __( '%s (%s)', 'page-generator-pro' ), 
                                                    ucfirst( $this->base->get_class( 'groups' )->get_status( $post_id ) ),
                                                    $this->base->get_class( 'groups' )->get_system( $post_id )
                                                ) . '
                                            </div>',
                        'cancel_generation' => '<a href="edit.php?post_type=' . $this->base->get_class( 'post_type' )->post_type_name . '&' . $this->base->plugin->name . '-action=cancel_generation&id=' . $post_id . '&type=content">' . $this->base->get_class( 'groups_ui' )->get_title( 'cancel_generation' ) . '</a>',
                    );
                }
                break;

            default:
                /**
                 * Define items to ouptput for a non-standard column on the Groups: Terms WP_List_Table.
                 *
                 * @since   1.5.9
                 *
                 * @param   string  $column_name    Column Name
                 * @param   int     $post_id        Group ID
                 */
                $items = apply_filters( 'page_generator_pro_groups_table_admin_columns_output', $items, $column_name, $post_id );
                break;
        }

        // If no items are defined for output, bail
        if ( empty( $items ) ) {
            return;
        }

        // Iterate through items, outputting
        foreach ( $items as $class => $item ) {
            echo '<span class="' . $class . '">' . $item . '</span><br />';
        }

    }

    /**
     * Adds Row Actions below the Title in the Groups WP_List_Table.
     *
     * @since   1.2.3
     *
     * @param   array   $actions    Row Actions
     * @return  array               Row Actions
     */
    public function admin_row_actions( $actions ) {

        global $post;

        // Bail if not a Groups CPT
        if ( get_post_type( $post ) != $this->base->get_class( 'post_type' )->post_type_name ) {
            return $actions;
        }

        // Add Duplicate Action
        $actions['duplicate'] = '<a href="edit.php?post_type=' . $this->base->get_class( 'post_type' )->post_type_name . '&' . $this->base->plugin->name . '-action=duplicate&id=' . $post->ID . '&type=content">' . $this->base->get_class( 'groups_ui' )->get_title( 'duplicate' ) . '</a>';
        
        /**
         * Filters the row actions to output on each Content Group in the Groups: Content WP_List_Table.
         *
         * @since   1.2.3
         *
         * @param   array       $actions                Row Actions
         * @param   WP_Post     $post                   Post
         */
        $actions = apply_filters( 'page_generator_pro_groups_table_admin_row_actions', $actions, $post );

        // Return
        return $actions;

    }

    /**
     * Checks if the Duplicate row action was clicked by the User, and if so
     * duplicates the given Group ID.
     *
     * @since   1.2.3
     */
    public function run_row_actions() {

        // Bail if we're not on a Groups screen
        if ( ! isset( $_REQUEST['post_type'] ) ) {
            return;
        }
        if ( sanitize_text_field( $_REQUEST['post_type'] ) != $this->base->get_class( 'post_type' )->post_type_name ) {
            return;
        }

        // If no action specified, return
        if ( ! isset( $_REQUEST[ $this->base->plugin->name . '-action' ] ) ) {
            return;
        }

        // Fetch action and group ID
        $action = sanitize_text_field( $_REQUEST[ $this->base->plugin->name . '-action' ] );
        $id = absint( $_REQUEST['id'] );

        // Run an action on the Group now
        $this->base->get_class( 'groups' )->run_action( $action, $id, true );

    }

    /**
     * Outputs the Javascript Overlay in the Footer
     *
     * @since   1.8.4
     */
    public function output_js_overlay() {

        // Load view
        include( $this->base->plugin->folder . 'views/admin/js-overlay.php' ); 

    }

}