<?php
/**
 * Unsplash API class
 * 
 * @package Page Generator Pro
 * @author  Tim Carr
 * @version 1.7.8
 */
class Page_Generator_Pro_Unsplash {

    /**
     * Holds the base object.
     *
     * @since   1.9.8
     *
     * @var     object
     */
    public $base;

    /**
     * Holds the Unsplash Access Key
     *
     * @since   1.7.8
     *
     * @var     string
     */
    private $access_key = 'b54338f4aa8813288213f6590a0db0fc0f7648fd842522e5bbe41cf9f3ca95c9';

    /**
     * Holds the API endpoint
     *
     * @since   1.7.8
     *
     * @var     string
     */
    private $endpoint = 'https://api.unsplash.com';

    /**
     * Constructor.
     *
     * @since   1.9.8
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

    }

    /**
     * Overrides the default Access Key with the supplied Access Key
     *
     * @since   1.7.8
     *
     * @param   string  $access_key     Access Key
     */
    public function set_access_key( $access_key ) {

        $this->access_key = $access_key;

    }

    /**
     * Gets a random photo based on the optional search parameters
     *
     * @see     https://unsplash.com/documentation#get-a-random-photo
     *
     * @since   1.7.8
     *
     * @param   string  $query          Search Term(s)
     * @param   string  $orientation    Orientation (landscape,portrait,squarish)
     * @param   int     $count          Number of Images to Return (1 - 30)
     * @param   string  $size           Image Size
     * @return  array   $images         Images | WP_Error
     */
    public function photos_random( $query = false, $orientation = false, $count = 1, $size = 'regular' ) {

        // Build array of arguments
        $args = array(
            'count'         => $count,
        );
        if ( $query != false ) {
            $args['query'] = sanitize_text_field( $query );
        }
        if ( $orientation != false ) {
            $args['orientation'] = sanitize_text_field( $orientation );
        }

        /**
         * Filters the API arguments to send to the photos/random endpoint
         * on the Unsplash API.
         *
         * @since   1.7.8
         *
         * @param   array   $args           API arguments
         * @param   string  $query          Search Terms
         * @param   string  $orientation    Orientation (landscape,portrait,squarish)
         * @param   int     $count          Number of Images to Return (1 - 30)
         * @param   string  $size           Image Size
         */
        $args = apply_filters( 'page_generator_pro_unsplash_photos_random_args', $args, $query, $orientation, $count, $size );
        
        // Run the query
        $results = $this->get( 'photos/random', $args );

        if ( is_wp_error( $results ) ) {
            return $results;
        }

        // Parse results
        $images = array();
        foreach ( $results as $photo ) {
            $images[] = array(
                'url'       => $photo->urls->{ $size }, // raw, full, regular, small, thumb
                'title'     => $photo->description,
            );
        }

        // Return array of images
        return $images;

    }

    /**
     * Private function to perform a GET request
     *
     * @since  1.0.0
     *
     * @param  string  $cmd        Command (required)
     * @param  array   $params     Params (optional)
     * @return mixed               WP_Error | object
     */
    private function get( $cmd, $params = array() ) {

        return $this->request( $cmd, 'get', $params );

    }

    /**
     * Private function to perform a POST request
     *
     * @since  1.7.8
     *
     * @param  string  $cmd        Command (required)
     * @param  array   $params     Params (optional)
     * @return mixed               WP_Error | object
     */
    private function post( $cmd, $params = array() ) {

        return $this->request( $cmd, 'post', $params );

    }

    /**
     * Private function to perform a PUT request
     *
     * @since  1.7.8
     *
     * @param  string  $cmd        Command (required)
     * @param  array   $params     Params (optional)
     * @return mixed               WP_Error | object
     */
    private function put( $cmd, $params = array() ) {

        return $this->request( $cmd, 'put', $params );

    }

    /**
     * Main function which handles sending requests to the Buffer API
     *
     * @since   1.7.8
     *
     * @param   string  $cmd        Command
     * @param   string  $method     Method (get|post)
     * @param   array   $params     Parameters (optional)
     * @return  mixed               WP_Error | object
     */
    private function request( $cmd, $method = 'get', $params = array() ) {

        // Build endpoint URL
        $url = $this->endpoint . '/' . $cmd;

        // Timeout
        $timeout = 5;

        /**
         * Defines the maximum time to allow the API request to run.
         *
         * @since   1.7.8
         *
         * @param   int     $timeout    Timeout, in seconds
         */
        $timeout = apply_filters( 'page_generator_pro_unsplash_request_timeout', $timeout );

        // Define headers
        $headers = array(
            'Accept'        => 'application/json',
            'Authorization' => 'Client-ID ' . $this->access_key,
            'Content-Type'  => 'application/json',
        );

        // Send request
        switch ( $method ) {
            /**
             * GET
             */
            case 'get':
                $result = wp_remote_get( $url, array(
                    'headers'   => $headers,
                    'body'      => ( $params != false ? $params : '' ),
                    'timeout'   => $timeout,
                ) );
                break;
            
            /**
             * POST
             */
            case 'post':
                $result = wp_remote_post( $url, array(
                    'headers'   => $headers,
                    'body'      => ( $params != false ? json_encode( $params ) : '' ),
                    'timeout'   => $timeout,
                ) );
                break;

            /**
             * PUT
             */
            case 'put':
                $result = wp_remote_post( $url, array(
                    'headers'   => $headers,
                    'method'    => 'PUT',
                    'body'      => ( $params != false ? json_encode( $params ) : '' ),
                    'timeout'   => $timeout,
                ) );
                break;
        }

        // If an error occured, return it now
        if ( is_wp_error( $result ) ) {
            return $result;
        }

        // Fetch HTTP response code and body
        $http_response_code = wp_remote_retrieve_response_code( $result );
        $body = wp_remote_retrieve_body( $result );

        // If the response code isn't valid, return an error now
        switch ( $http_response_code ) {
            case 403:
                return new WP_Error( 'page_generator_pro_unsplash_request_error', sprintf( __( 'Unsplash API Error: %s', 'page-generator-pro' ), $body ) );
                break;

            // @TODO Handle other errors here
        }

        // Decode the JSON string
        $body = json_decode( $body );
        
        // Bail if an error occured
        if ( isset( $body->errors ) ) {
            // Build error message
            $message = implode( "\n", $body->errors );
            return new WP_Error( 'page_generator_pro_unsplash_request_error', $message ); 
        }

        // All OK, return response
        return $body;

    }

    /**
     * Returns the singleton instance of the class.
     *
     * @since       1.1.6
     * @deprecated  1.9.8
     *
     * @return      object Class.
     */
    public static function get_instance() {

        // Define class name
        $name = 'unsplash';

        // Warn the developer that they shouldn't use this function.
        _deprecated_function( __FUNCTION__, '1.9.8', 'Page_Generator_Pro()->get_class( \'' . $name . '\' )' );

        // Return the class
        return Page_Generator_Pro()->get_class( $name );

    }

}