<?php
/**
 * Shortcode class
 * 
 * @package Page_Generator_Pro
 * @author  Tim Carr
 * @version 1.0.0
 */
class Page_Generator_Pro_Shortcode {

    /**
     * Holds the base object.
     *
     * @since   1.2.1
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   1.0.0
     *
     * @param   object $base    Base Plugin Class
     */
    public function __construct( $base ) {

        // Store base class
        $this->base = $base;

        // Hooks and Filters
        add_filter( 'wp_get_custom_css', array( $this, 'load_css' ), 10, 2 );
        add_filter( 'the_content', array( $this, 'maybe_change_css_prefix_content' ) );

        // Register shortcodes
        $this->add_shortcodes( false );

    }

    /**
     * Registers the shortcodes used by this plugin, depending on whether we're running
     * content generation for a Group or not.
     *
     * @since   1.2.0
     *
     * @param   bool    $generating_group   Generating Group
     */
    public function add_shortcodes( $generating_group = false ) {

        // Get shortcodes
        $shortcodes = $this->get_shortcodes();

        // Bail if no shortcodes are available
        if ( ! is_array( $shortcodes ) || count( $shortcodes ) == 0 ) {
            return;
        }

        // Get CSS Prefix
        $css_prefix = $this->get_css_prefix();

        // Iterate through shortcodes, registering them
        foreach ( $shortcodes as $shortcode => $properties ) {
            // Skip if this shortcode should only be registered on content generation
            if ( $generating_group && ! $properties['register_on_generation_only'] ) {
                continue;
            }

            // Register the shortcode
            add_shortcode( $this->base->plugin->name . '-' . $shortcode, array( $this, $properties['render_callback'] ) );

            // If a CSS Prefix is specified, and this is the Related Links shortcode, register the actual CSS prefix
            // as an additional shortcode
            if ( $css_prefix == $this->base->plugin->name ) {
                continue;
            }
            if ( $shortcode != 'related-links' ) {
                continue;
            }
            add_shortcode( $css_prefix . '-' . $shortcode, array( $this, $properties['render_callback'] ) );
               
        }

        /**
         * Add any shortcodes here, using add_shortcode(), that should be processed using do_shortcode()
         * when a Page is Generated.  Do NOT register shortcodes that you do NOT want processing into HTML output
         * on the resulting Pages.
         *
         * @since   1.9.8
         *
         * @param   bool    $generating_group   Generating Pages from Group (true|false)
         */
        do_action( 'page_generator_pro_shortcode_add_shortcodes', $generating_group );

    }

    /**
     * Appends the contents of this Plugin's frontend.css file to the WordPress
     * Theme Customizer Additional CSS.
     *
     * We do this instead of enqueueing CSS to avoid what people believe is
     * the 'footprint' problem.
     *
     * @since   2.0.4
     *
     * @param   string  $customizer_css     Customizer CSS
     * @param   string  $stylesheet         Stylesheet URI
     * @return  string                      Customizer CSS
     */
    public function load_css( $customizer_css, $stylesheet ) {

        // Bail if in the admin interface
        if ( function_exists( 'is_admin' ) && is_admin() ) {
            return $customizer_css;
        }

        // Fetch Frontend CSS
        $plugin_css = file_get_contents( $this->base->plugin->folder . '/assets/css/frontend.css' );

        // Change prefixes, if required
        $plugin_css = $this->change_css_prefix( $plugin_css );

        // If the Customizer CSS already contains the Frontend CSS, bail
        if ( strpos( $customizer_css, $plugin_css ) ) {
            return $customizer_css;
        }

        // Append CSS
        return $customizer_css . "\n" . $plugin_css;
    
    }

    /**
     * Replaces CSS prefix in the content
     *
     * @since   2.0.3
     *
     * @param   string  $content    Content
     * @return  string              Content
     */
    public function maybe_change_css_prefix_content( $content ) {

        // Change CSS prefix
        $content = $this->change_css_prefix( $content );

        // Return
        return $content;

    }

    /**
     * Returns an array comprising of plugin specific Shortcodes,
     * and their attributes.
     *
     * This is used by both TinyMCE and Gutenberg, so that Shortcodes
     * and Blocks are registered.
     *
     * @since   2.0.5
     */
    public function get_shortcodes() {

        $shortcodes = array(
            /**
             * Google Map
             */
            'google-map' => array(
                // Gutenberg Block Details
                'title'     => __( 'Google Map', $this->base->plugin->name ),
                'icon'      => '',
                'category'  => 'layout',
                'keywords'  => array(
                    __( 'Google Map', $this->base->plugin->name ),
                    __( 'Google', $this->base->plugin->name ),
                    __( 'Map', $this->base->plugin->name ),
                ),

                // Attributes
                'attributes'    => array(
                    'maptype'       => array(
                        'label'         => __( 'Map Type', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                    ),
                    'location'       => array(
                        'label'         => __( 'Location', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                        'choices'       => false,
                    ),
                    'destination'   => array(
                        'label'         => __( 'Destination', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                        'choices'       => false,
                    ),
                    'country_code'  => array(
                        'label'         => __( 'Country Code', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                        'choices'       => false,
                    ),
                    'term'          => array(
                        'label'         => __( 'Term', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                        'choices'       => false,
                    ),
                    'height'        => array(
                        'label'         => __( 'Height (px)', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                        'choices'       => false,
                    ),
                    'zoom'          => array(
                        'label'         => __( 'Zoom Level', $this->base->plugin->name ),
                        'type'          => 'text',
                        'description'   => '',
                        'choices'       => false,
                    ),
                ),

                // Default Values
                'default_values' => array(
                    'maptype'       => 'roadmap',
                    'location'      => '',
                    'destination'   => '',
                    'country_code'  => '',
                    'term'          => '',
                    'height'        => 250,
                    'zoom'          => 14,
                ),

                // Register when Generation is running only
                'register_on_generation_only' => true,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'google_map',
            ),

            /**
             * Media Library
             */
            'media-library' => array(
                // Gutenberg Block Details
                'title'     => __( 'Media Library', $this->base->plugin->name ),

                // Default Values
                'default_values' => array(
                    'title'         => false,
                    'caption'       => false,
                    'alt'           => false,
                    'description'   => false,
                    'ids'           => false,
                    'min_id'        => false,
                    'max_id'        => false,
                    'size'          => 'medium',
                    'alt_tag'       => false,
                ),

                // Register when Generation is running only
                'register_on_generation_only' => true,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'media_library',
            ),

            /**
             * Related Links
             */
            'related-links' => array(
                // Gutenberg Block Details
                'title'     => __( 'Related Links', $this->base->plugin->name ),
                
                // Register when Generation is running only
                'register_on_generation_only' => false,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'related_links',
            ),

            /**
             * Unsplash
             */
            'unsplash' => array(
                // Gutenberg Block Details
                'title'     => __( 'Unsplash', $this->base->plugin->name ),

                // Default Values
                'default_values' => array(
                    'term'          => false,
                    'title'         => false,
                    'caption'       => false,
                    'alt_tag'       => false,
                    'orientation'   => false,
                    'size'          => 'full',
                ),
                
                // Register when Generation is running only
                'register_on_generation_only' => true,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'unsplash',
            ),

            /**
             * Wikipedia
             */
            'wikipedia' => array(
                // Gutenberg Block Details
                'title'     => __( 'Wikipedia', $this->base->plugin->name ),

                // Default Values
                'default_values' => array(
                    'term'      => '',
                    'language'  => 'en',
                    'sections'  => 2, // Number of wikipedia sections to include
                ),
                
                // Register when Generation is running only
                'register_on_generation_only' => true,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'wikipedia',
            ),

            /**
             * Yelp
             */
            'yelp' => array(
                // Gutenberg Block Details
                'title'     => __( 'Yelp', $this->base->plugin->name ),

                // Default Values
                'default_values' => array(
                    'term'      => '',
                    'location'  => '',
                    'limit'     => 5,
                ),
                
                // Register when Generation is running only
                'register_on_generation_only' => true,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'yelp',
            ),

            /**
             * YouTube
             */
            'youtube' => array(
                // Gutenberg Block Details
                'title'     => __( 'YouTube', $this->base->plugin->name ),

                // Default Values
                'default_values' => array(
                    'term'          => '',
                    'location'      => 0,
                ),
                
                // Register when Generation is running only
                'register_on_generation_only' => true,

                // Function to call when rendering the shortcode on the frontend
                'render_callback' => 'youtube',
            ),
        );

        /**
         * Defines an array comprising of plugin specific Shortcodes,
         * and their attributes.
         *
         * This is used by both TinyMCE and Gutenberg, so that Shortcodes
         * and Blocks are registered.
         *
         * @since   2.0.5
         *
         * @param   array   $shortcodes     Registered Shortcodes
         */
        $shortcodes = apply_filters( 'page_generator_pro_shortcode_get_shortcodes', $shortcodes );

        // Return
        return $shortcodes;

    }

    /**
     * Helper method to fetch the specified shortcode from get_shortcodes()
     *
     * @since   2.0.6
     *
     * @param   string  $shortcode_name     Shortcode Name
     * @return  mixed                       WP_Error | array
     */
    public function get_shortcode( $shortcode_name ) {

        // Fetch all shortcodes
        $shortcodes = $this->get_shortcodes();

        // Bail if the given shortcode name has not been registered
        if ( ! is_array( $shortcodes ) || count( $shortcodes ) == 0 || ! isset( $shortcodes[ $shortcode_name ] ) ) {
            return new WP_Error( 
                'page_generator_pro_shortcode_get_shortcode', 
                sprintf(
                    __( 'The %s shortcode could not be found, as it is not registered in get_shortcodes()', 'page-generator-pro' ),
                    $shortcode_name
                )
            );
        }  

        // Return the shortcode
        return $shortcodes[ $shortcode_name ]; 

    }

    /**
     * Google Maps Shortcode
     *
     * @since  1.0.0
     *
     * @param  array   $atts   Shortcode Attributes
     * @return string          Output
     */
    public function google_map( $atts ) {

        // Fetch shortcode
        $shortcode = $this->get_shortcode( 'google-map' );

        // Bail if an error occured
        if ( is_wp_error( $shortcode ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Google Maps Shortcode Error: %s', 'page-generator-pro' ), $shortcode->get_error_message() );
            }
            return '';
        }

        // Parse shortcode attributes, defining fallback defaults if required
        $atts = shortcode_atts( $shortcode['default_values'], $atts, $this->base->plugin->name . '-google-map' );

        // Build iframe arguments and determine endpoint
        $args = false;
        switch ( $atts['maptype'] ) {

            /**
             * Street View
             */
            case 'streetview':
                // Get latitude and longitude
                $lat_lng = false;
                $result = $this->base->get_class( 'georocket' )->get_geocode( $atts['location'] . ', ' . $atts['country_code'], $this->base->licensing->get_license_key() );

                // Bail if errors occured
                if ( is_wp_error( $result ) ) {
                    if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                        return sprintf( __( 'Google Maps Shortcode Error: %s', 'page-generator-pro' ), $result->get_error_message() );
                    }

                    return '';
                }

                if ( ! $result->data ) {
                    if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                        return sprintf( __( 'Google Maps Shortcode Error: %s', 'page-generator-pro' ), 'No Data in Geocode Response' );
                    }

                    return '';
                }

                // If here, we have a latitude and longitude
                $endpoint = $atts['maptype'];
                $args = array(
                    'location' => $result->data->latitude . ',' . $result->data->longitude,
                );
                break;

            /**
             * Directions
             */
            case 'directions':
                $endpoint = $atts['maptype'];
                $args = array(
                    'origin'        => $atts['location'],
                    'destination'   => $atts['destination'],
                );
                break;

            /**
             * Road Map or Satellite
             */
            default:
                $endpoint = 'place';
                $args = array(
                    'q'         => ( ! empty( $atts['term'] ) ? $atts['term'] . ' in ' . $atts['location'] : $atts['location'] ),
                    'zoom'      => $atts['zoom'],
                    'maptype'   => $atts['maptype'],
                );
                break;

        }

        // Fetch API Key
        // For iframe embeds, there is no usage limit, so we can use the Plugin API Key
        $args['key'] = $this->base->get_class( 'geo' )->google_maps_embed_api_key;

        // Build URL
        $url = 'https://www.google.com/maps/embed/v1/' . $endpoint . '?' . http_build_query( $args );

        /**
         * Filter the Google Maps iFrame URL, before output.
         *
         * @since   2.0.4
         *
         * @param   string  $url        URL with Arguments
         * @param   array   $atts       Shortcode Attributes
         * @param   array   $args       URL Arguments
         * @param   string  $endpoint   URL Endpoint
         */
        $url = apply_filters( 'page_generator_pro_shortcode_google_maps_url', $url, $atts, $args, $endpoint );

        // Build HTML using the URL
        $html = '<iframe class="page-generator-pro-map" width="100%" height="' . $atts['height'] . '" frameborder="0" style="border:0" src="' . $url . '" allowfullscreen></iframe>';

        /**
         * Filter the Google Maps HTML output, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $html   HTML Output
         * @param   array   $atts   Shortcode Attributes
         */
        $html = apply_filters( 'page_generator_pro_shortcode_google_maps', $html, $atts );

        // Return
        return $html;

    }

    /**
     * Media Library Shortcode
     *
     * @since   1.7.9
     *
     * @param   array   $atts   Shortcode Attributes
     * @return  string          Output
     */
    public function media_library( $atts ) {

        // Fetch shortcode
        $shortcode = $this->get_shortcode( 'media-library' );

        // Bail if an error occured
        if ( is_wp_error( $shortcode ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Media Library Shortcode Error: %s', 'page-generator-pro' ), $shortcode->get_error_message() );
            }
            return '';
        }

        // Parse shortcode attributes, defining fallback defaults if required
        $atts = shortcode_atts( $shortcode['default_values'], $atts, $this->base->plugin->name . '-media-library' );

        // Get Image HTML
        $html = $this->base->get_class( 'media_library' )->get_random_image_html_tag( $atts );

        /**
         * Filter the Media Library HTML output, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $html   HTML Output
         * @param   array   $atts   Shortcode Attributes
         */
        $html = apply_filters( 'page_generator_pro_shortcode_media_library', $html, $atts );

        // Return
        return $html;
        
    }

    /**
     * Unsplash Shortcode
     *
     * @since   1.7.8
     *
     * @param   array   $atts   Shortcode Attributes
     * @return  string          Output
     */
    public function unsplash( $atts ) {

        // Fetch shortcode
        $shortcode = $this->get_shortcode( 'unsplash' );

        // Bail if an error occured
        if ( is_wp_error( $shortcode ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Unsplash Shortcode Error: %s', 'page-generator-pro' ), $shortcode->get_error_message() );
            }
            return '';
        }

        // Parse shortcode attributes, defining fallback defaults if required
        $atts = shortcode_atts( $shortcode['default_values'], $atts, $this->base->plugin->name . '-unsplash' );

        // Get Post ID
        $post_id = $this->get_post_or_group_id();

        // Import an image into WordPress now
        // Setup Unsplash API instance
        $api = $this->base->get_class( 'unsplash' );

        // If an Unsplash Access Key has been specified, use it instead of the class default.
        $access_key = $this->base->get_class( 'settings' )->get_setting( $this->base->plugin->name . '-unsplash', 'access_key' );
        if ( ! empty( $access_key ) ) {
            $api->set_access_key( $access_key );
        }

        // Run images query
        $images = $api->photos_random( $atts['term'], $atts['orientation'], 1, $atts['size'] );
        if ( is_wp_error( $images ) || ! is_array( $images ) || count( $images ) == 0 ) {
            // Couldn't fetch an image, so don't show an image at all.
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Unsplash Shortcode Error: %s', 'page-generator-pro' ), $images->get_error_message() );
            }

            return '';
        }

        // Pick an image at random from the resultset
        if ( count( $images ) == 1 ) {
            $image_index = 0;
        } else {
            $image_index = rand( 0, ( count( $images ) - 1 ) );
        }   

        // Import the first image
        $image_id = $this->base->get_class( 'import' )->import_remote_image( 
            $images[ $image_index ]['url'], 
            $post_id, 
            ( ! $atts['title'] ? $images[ $image_index ]['title'] : $atts['title'] ), // title
            ( ! $atts['caption'] ? $images[ $image_index ]['title'] : $atts['caption'] ), // caption
            ( ! $atts['alt_tag'] ? $images[ $image_index ]['title'] : $atts['alt_tag'] ) // alt_tag
        );

        // Bail if an error occured
        if ( is_wp_error( $image_id ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Unsplash Shortcode Error: %s', 'page-generator-pro' ), $image_id->get_error_message() );
            }

            return '';
        }

        // Get the image HTML tag
        $html = wp_get_attachment_image( $image_id, 'full' );

        /**
         * Filter the Unsplash HTML output, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $html       HTML Output
         * @param   array   $atts       Shortcode Attributes
         * @param   int     $image_id   WordPress Media Library Image ID
         * @param   array   $images     Unsplash Image Results
         */
        $html = apply_filters( 'page_generator_pro_shortcode_unsplash', $html, $atts, $image_id, $images );

        // Return
        return $html;

    }

    /**
     * Related Links Shortcode
     *
     * @since   1.5.9
     *
     * @param   array   $atts   Shortcode Attributes
     * @return  string          Output
     */
    public function related_links( $atts ) {

        global $post;

        // Bail if we're in the WordPress Admin
        if ( is_admin() ) {
            return '';
        }

        // Get the Group ID which generated this content
        $group_id = absint( get_post_meta( $post->ID, '_page_generator_pro_group', true ) );

        // Define default shortcode attributes
        $defaults = array(
            // Output and Group ID
            'limit'         => 5,
            'group_id'      => $group_id,

            // Post
            'post_type'     => $post->post_type,
            'post_status'   => 'publish',
            'post__not_in'  => array( $post->ID ),

            // Order
            'orderby'       => 'rand',
            'order'         => 'ASC',
            
            // Author
            'author'        => '',
        );
        foreach ( $this->base->get_class( 'common' )->get_taxonomies() as $taxonomy ) {
            $defaults[ $taxonomy->name ] = '';
        }

        /**
         * Filter the Related Links Shortcode Default Attributes.
         *
         * @since   1.0.0
         *
         * @param   array       $defaults   Default Attributes
         * @param   array       $atts       Shortcode Attributes
         * @param   WP_Post     $post       WordPress Post
         */
        $defaults = apply_filters( 'page_generator_pro_shortcode_related_links_defaults', $defaults, $atts, $post );

        // Parse shortcode attributes, using defaults if required
        $atts = shortcode_atts( $defaults, $atts, $this->base->plugin->name . '-related-links' );

        // If certain attributes are empty, but require a value, set them now
        if ( empty( $atts['group_id'] ) ) {
            $atts['group_id'] = $group_id;
        }
        if ( empty( $atts['post_type'] ) ) {
            $atts['post_type'] = $post->post_type;
        }

        // Start building the WP_Query arguments
        $args = array(
            'post_type'     => $atts['post_type'],
            'post_status'   => $atts['post_status'],
            'post__not_in'  => $atts['post__not_in'],
            'posts_per_page'=> absint( $atts['limit'] ),
            'orderby'       => $atts['orderby'],
            'order'         => $atts['order'],
            'meta_query'    => array(
                array(
                    'key'   => '_page_generator_pro_group',
                    'value' => absint( $atts['group_id'] ),
                ),
            ),

            // For performance, just return the Post ID and don't update meta or term caches
            'fields'                => 'ids',
            'cache_results'         => false,
            'update_post_meta_cache'=> false,
            'update_post_term_cache'=> false,
        );

        // Add Author constraint to WP_Query, if defined
        if ( ! empty( $atts['author'] ) ) {
            $args['author'] = absint( $atts['author'] );
        }

        // Add Taxonomy Term constraints to WP_Query, if defined
        // Fetch Taxonomies for this Post Type
        $taxonomies = $this->base->get_class( 'common' )->get_post_type_taxonomies( $args['post_type'] );
        if ( is_array( $taxonomies ) && count( $taxonomies ) > 0 ) {
            $args['tax_query'] = array();
            foreach ( $taxonomies as $taxonomy ) {
                // Skip if no constraints defined for this Taxonomy
                if ( ! isset( $atts[ $taxonomy->name ] ) ) {
                    continue;
                }
                if ( empty( $atts[ $taxonomy->name ] ) ) {
                    continue;
                }

                // Build array of Terms, checking if they're numeric (i.e. all Term IDs)
                $terms = array_map( 'trim', explode( ',', $atts[ $taxonomy->name ] ) );
                $terms_are_numeric = true;
                foreach ( $terms as $term ) {
                    if ( ! is_numeric( $term ) ) {
                        $terms_are_numeric = false;
                        break;
                    }
                }

                // Add constraint
                $args['tax_query'][] = array(
                    'taxonomy'  => $taxonomy->name,
                    'field'     => ( $terms_are_numeric ? 'term_id' : 'name' ),
                    'terms'     => $terms,
                );
            }
        }

        // Fetch other Pages / Posts generated in this Group based on the supplied arguments
        $posts = new WP_Query( $args );

        // If no other Pages / Posts found, bail
        if ( count( $posts->posts ) == 0 ) {
            return '';
        }

        // Build link list
        $html = '<ul class="page-generator-pro-related-links">';
        foreach ( $posts->posts as $post_id ) {
            $html .= '<li><a href="' . get_permalink( $post_id ) . '" title="' . get_the_title( $post_id ) . '">' . get_the_title( $post_id ) . '</a></li>';
        }
        $html .= '</ul>';

        /**
         * Filter the Related Links Shortcode HTML output, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $html   HTML Output
         * @param   array   $atts   Shortcode Attributes
         * @param   array   $posts  Related Posts
         * @param   WP_Post $post   WordPress Post the shortcode is used on
         */
        $html = apply_filters( 'page_generator_pro_shortcode_related_links', $html, $atts, $posts, $post );

        // Return
        return $html;

    }

    /**
     * Wikipedia Shortcode
     *
     * @since  1.0.0
     *
     * @param  array   $atts   Shortcode Attributes
     * @return string          Output
     */
    public function wikipedia( $atts ) {

        // Check DOMDocument installed
        if ( ! class_exists( 'DOMDocument' ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return __( 'Wikipedia Shortcode Error: PHP DOMDocument Extension is not installed. This is required to parse Wikipedia content.', 'page-generator-pro' );
            }

            return '';
        }

        // Fetch shortcode
        $shortcode = $this->get_shortcode( 'wikipedia' );

        // Bail if an error occured
        if ( is_wp_error( $shortcode ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Wikipedia Error: %s', 'page-generator-pro' ), $shortcode->get_error_message() );
            }
            return '';
        }

        // Parse shortcode attributes, defining fallback defaults if required
        $atts = shortcode_atts( $shortcode['default_values'], $atts, $this->base->plugin->name . '-wikipedia' );

        // Check if an article exists on Wikipedia for the given term
        $response = wp_remote_get( 'http://' . $atts['language'] . '.wikipedia.org/w/api.php?section=0&action=parse&page=' . $atts['term'] . '&format=json&prop=text&redirects' );

        // Bail if an error occured
        if ( is_wp_error( $response ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Wikipedia Shortcode Error: %s', 'page-generator-pro' ), $response->get_error_message() );
            }

            return '';
        }
        if ( $response['response']['code'] != 200 ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Wikipedia Shortcode Error: %s', 'page-generator-pro' ), $response['response']['code'] );
            }

            return '';
        }

        // Check status
        $body = wp_remote_retrieve_body( $response );
        $check = json_decode( $body );

        // Bail if an error was received from Wikipedia
        if ( isset( $check->error ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Wikipedia Shortcode Error: %s', 'page-generator-pro' ), $check->error->code . ': ' . $check->error->info );
            }

            return '';
        }
        
        // Get data from Wikipedia
        $url = 'http://' . $atts['language'] . '.wikipedia.org/wiki/' . str_replace(' ', '_', $atts['term'] );
        $response = wp_remote_get( $url );

        // Bail if an error occured
        if ( is_wp_error( $response ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Wikipedia Shortcode Error: %s', 'page-generator-pro' ), $response->get_error_message() );
            }
        }
        if ( $response['response']['code'] != 200 ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Wikipedia Shortcode Error: %s', 'page-generator-pro' ), $response['response']['code'] );
            }

            return '';
        }

        // Get body
        $body = wp_remote_retrieve_body( $response );
        if ( empty( $body ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return __( 'Wikipedia Shortcode Error: No Wikipedia content found', 'page-generator-pro' );
            }

            return '';
        }

        // Parse into DOMDocument
        $dom = new DOMDocument();
        $dom->preserveWhiteSpace = false; 

        // Convert encoding to UTF-8 if php-mbstring is installed
        if ( function_exists( 'mb_convert_encoding' ) ) {
            @$dom->loadHTML( mb_convert_encoding( $body, 'HTML-ENTITIES', 'UTF-8' ) );
        } else {
            // Cannot guarantee this works as mb_convert_encoding is not available
            @$dom->loadHTML( $body );
        }

        // Extract paragraphs that contain the main content
        $paragraphs = $dom->getElementById( 'mw-content-text' )->getElementsByTagName( 'p' );
        
        // Build paragraphs
        $x = 1;
        $build = '';
        foreach( $paragraphs as $paragraph ) {
            $possible_paragraph = trim( $paragraph->nodeValue );

            // Skip if the paragraph is empty
            if ( empty( $possible_paragraph ) ) {
                continue;
            }

            // Skip if this entire paragraph's nodeValue matches the keyword
            // e.g. the city name
            if ( strpos( $atts['term'], $possible_paragraph ) !== false ) {
                continue;
            }

            // Skip if the paragraph starts with certain characters
            if ( strpos( $possible_paragraph, '[[' ) !== false ) {
                continue;
            }

            if ( $x <= $atts['sections'] ) {
                $build .= "<p>" . $possible_paragraph . "</p>";
                $x++;
            } else {
                break;
            }
        }

        // Check WikiPedia returned content in our build process
        if ( empty( $build ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return __( 'Wikipedia Shortcode Error: Generated Wikipedia content empty', 'page-generator-pro' );
            }

            return '';
        }

        // Build HTML
        $html = '<div class="' . $this->base->plugin->name . '-wikipedia">' . $build . '</div>';

        /**
         * Filter the Wikipedia Shortcode HTML output, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $html   HTML Output
         * @param   array   $atts   Shortcode Attributes
         * @param   string  $build  Wikipedia Content
         */
        $html = apply_filters( 'page_generator_pro_shortcode_wikipedia', $html, $atts, $build );

        // Return
        return $html;

    }

    /**
     * Yelp Shortcode
     *
     * @since 1.0.0
     *
     * @param  array   $atts   Shortcode Attributes
     * @return string          Output
     */
    public function yelp( $atts ) {

        // Fetch shortcode
        $shortcode = $this->get_shortcode( 'yelp' );

        // Bail if an error occured
        if ( is_wp_error( $shortcode ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Yelp Error: %s', 'page-generator-pro' ), $shortcode->get_error_message() );
            }
            return '';
        }

        // Parse shortcode attributes, defining fallback defaults if required
        $atts = shortcode_atts( $shortcode['default_values'], $atts, $this->base->plugin->name . '-yelp' );

        // Send request to Yelp API
        $yelp = $this->base->get_class( 'yelp' );
        $results = $yelp->businesses_search( $atts['term'], $atts['location'], $atts['limit'] );
        
        // Check for errors from the Yelp API
        if ( is_wp_error( $results ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'Yelp Shortcode Error: %s', 'page-generator-pro' ), $results->get_error_message() );
            }

            // Just return a blank string
            return '';
        }

        // Check if any businesses were found
        if ( ! isset( $results->businesses ) || count( $results->businesses ) == 0 ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return __( 'Yelp Shortcode Error: No businesses found', 'page-generator-pro' );
            }

            return '';
        }

        // Iterate through results, building HTML
        $html = '<div class="' . $this->base->plugin->name . '-yelp">';
        foreach ( $results->businesses as $count => $business ) {

            $html .= '<div class="business">
                <div class="name">' . $business->name . '</div>
                <div class="rating rating">
                    <div class="rating-stars rating-stars-' . str_replace( '.', '-', $business->rating ) . '"></div>
                    ' . $business->review_count . ' ' . ( $business->review_count == 1 ? __( 'review', 'page-generator-pro' ) : __( 'reviews', 'page-generator-pro' ) ) . '
                </div>
                <div class="categories">';
            
            // Categories
            $total_categories = count( $business->categories );
            foreach ( $business->categories as $cCount => $category ) {
                $html .= $category->title;
                if ( ( $cCount + 1 ) != $total_categories ) {
                    $html .= ', ';
                }
            }
                
            $html .= '
                </div>
                <div class="phone">' . $business->phone . '</div>
                
                <div class="address">';
            
            // Address
            $total_address_lines = count( $business->location->display_address );
            foreach ( $business->location->display_address as $aCount => $address ) {
                $html .= $address;
                if ( ( $aCount + 1 ) != $total_address_lines ) {
                    $html .= ', ';
                }
            }
            
            $html .= '
                </div>
            </div>';
            
            // Check if limit reached
            if ( ( $count + 1 ) == $atts['limit'] ) {
                break;
            }
        }
        
        /**
         * Filter the Yelp Shortcode HTML output, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $html   HTML Output
         * @param   array   $atts   Shortcode Attributes
         */
        $html = apply_filters( 'page_generator_pro_shortcode_yelp', $html, $atts );

        // Add Yelp logo. This is required to meet the display requirements below, which is why this is done after filtering
        // http://www.yelp.co.uk/developers/getting_started/display_requirements
        $html .= '<a href="https://www.yelp.com" target="_blank"><img src="https://s3-media1.ak.yelpcdn.com/assets/2/www/img/55e2efe681ed/developers/yelp_logo_50x25.png" /></a>
        </div>';

        // Return
        return $html;

    }

    /**
     * YouTube Shortcode
     *
     * @since   1.2.0
     *
     * @param   array   $atts   Shortcode Attributes
     * @return  string          Output
     */
    public function youtube( $atts ) {

        // Fetch shortcode
        $shortcode = $this->get_shortcode( 'youtube' );

        // Bail if an error occured
        if ( is_wp_error( $shortcode ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'YouTube Error: %s', 'page-generator-pro' ), $shortcode->get_error_message() );
            }
            return '';
        }

        // Parse shortcode attributes, defining fallback defaults if required
        $atts = shortcode_atts( $shortcode['default_values'], $atts, $this->base->plugin->name . '-youtube' );

        // If our term is a location, get its latitude and longitude now
        $lat_lng = false;
        if ( $atts['location'] ) {
            $result = $this->base->get_class( 'georocket' )->get_geocode( $atts['location'], $this->base->licensing->get_license_key() );
            
            if ( ! is_wp_error( $result ) && $result->success && $result->data != false ) {
                $lat_lng = array(
                    'latitude'  => $result->data->latitude,
                    'longitude' => $result->data->longitude,
                );
            }
        }

        // Setup YouTube instance
        $youtube = $this->base->get_class( 'youtube' );
        
        // If a YouTube Data API key has been specified, use it instead of the class default.
        $youtube_data_api_key = $this->base->get_class( 'settings' )->get_setting( $this->base->plugin->name . '-google', 'youtube_data_api_key' );
        if ( ! empty( $youtube_data_api_key ) ) {
            $youtube->api_key = $youtube_data_api_key;
        }

        // Run query
        $videos = $youtube->search( $atts['term'], $lat_lng );
        if ( is_wp_error( $videos ) || ! is_array( $videos ) || count( $videos ) == 0 ) {
            // Couldn't fetch videos from YouTube
            // If a location was specified, remove it and try again
            if ( $lat_lng != false ) {
                $videos = $youtube->search( $atts['term'], false );
            }
        }

        // Bail if an error occured
        if ( is_wp_error( $videos ) ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return sprintf( __( 'YouTube Shortcode Error: %s', 'page-generator-pro' ), $videos->get_error_message() );
            }

            return '';
        }

        // Bail if no videos found
        if ( ! is_array( $videos ) || count( $videos ) == 0 ) {
            if ( defined( 'PAGE_GENERATOR_PRO_DEBUG' ) && PAGE_GENERATOR_PRO_DEBUG === true ) {
                return __( 'YouTube Shortcode Error: No videos found', 'page-generator-pro' );
            }

            return '';
        }

        // Pick a video at random from the resultset
        if ( count( $videos ) == 1 ) {
            $video_index = 0;
        } else {
            $video_index = rand( 0, ( count( $videos ) - 1 ) );
        }

        // Get video URL
        $url = $videos[ $video_index ]['url'];

        /**
         * Filter the YouTube Shortcode URL, before returning.
         *
         * @since   1.0.0
         *
         * @param   string  $url            YouTube URL
         * @param   array   $atts           Shortcode Attributes
         * @param   array   $videos         Video Results
         * @param   int     $video_index    Video Index
         */
        $url = apply_filters( 'page_generator_pro_shortcode_youtube', $url, $atts, $videos, $video_index );

        // Return
        return $url;

    }

    /**
     * Helper function to try and fetch either the Generated Post ID
     * or Group ID (if no Generated Post ID is available).
     *
     * This is then used by Shortcodes to store Attachments against
     * the given Post ID / Group ID.
     *
     * @since   1.9.7
     *
     * @return  int     Post or Group ID
     */
    private function get_post_or_group_id() {

        global $post, $page_generator_pro_group_id;

        if ( ! empty( $post ) ) {
            return $post->ID;
        }

        if ( ! empty( $page_generator_pro_group_id ) ) {
            return $page_generator_pro_group_id;
        }

        return 0;

    }

/**
     * Returns the CSS prefix to use.
     *
     * @since   2.0.3
     *
     * @return  string  CSS Prefix
     */
    private function get_css_prefix() {

        // Get prefix
        $css_prefix = trim( $this->base->get_class( 'settings' )->get_setting( $this->base->plugin->name . '-general', 'css_prefix' ) );
        
        // Fallback to plugin name if no prefix specified
        if ( empty( $css_prefix ) ) {
            $css_prefix = $this->base->plugin->name;
        }

        /**
         * Returns the CSS prefix to use.
         *
         * @since   2.0.3
         *
         * @param   string    $css_prefix   CSS Prefix to use
         */
        $css_prefix = apply_filters( 'page_generator_pro_shortcode_get_css_prefix', $css_prefix );

        // Return
        return $css_prefix;

    }

    /**
     * Changes the default Plugin CSS Prefix for the one specified in the Plugin Settings
     *
     * @since   2.0.3
     *
     * @param   string  $content    Content
     * @return  string              Amended Content
     */
    private function change_css_prefix( $content ) {

        // Get CSS Prefix
        $css_prefix = $this->get_css_prefix();

        // Bail if it matches the Plugin Name
        if ( $css_prefix == $this->base->plugin->name ) {
            return $content;
        }

        // Replace prefix
        $content = str_replace( $this->base->plugin->name, $css_prefix, $content );

        // Return
        return $content;

    }


    /**
     * Returns the singleton instance of the class.
     *
     * @since       1.1.6
     * @deprecated  1.9.8
     *
     * @return      object Class.
     */
    public static function get_instance() {

        // Define class name
        $name = 'shortcode';

        // Warn the developer that they shouldn't use this function.
        _deprecated_function( __FUNCTION__, '1.9.8', 'Page_Generator_Pro()->get_class( \'' . $name . '\' )' );

        // Return the class
        return Page_Generator_Pro()->get_class( $name );

    }

}